package com.winhc.repal.controller;

import com.winhc.common.base.BeanResponse;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.model.base.VOPage;
import com.winhc.repal.model.dto.*;
import com.winhc.repal.model.vo.*;
import com.winhc.repal.service.RepalBillService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;



/**
 * @description 回款宝账款
 * @author Generator
 * @date 2020-12-15
 */
@Api(tags = "回款宝账款")
@RestController
@RequestMapping(value = "/repalBill")
public class RepalBillController {

    @Autowired
    private RepalBillService repalBillService;


    @ApiOperation("获取账本下的账款列表(应收or应付)")
    @RequestMapping(value = "/page", method = RequestMethod.GET)
    public BeanResponse<RepalBillInfoVO> getRepalBillPage(GetRepalBillPageDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.getRepalBillPage(dto));
    }

    @ApiOperation("搜索账款")
    @RequestMapping(value = "/search", method = RequestMethod.GET)
    public BeanResponse<VOPage<RepalBillPageVO>> searchBill(SearchBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.searchBill(dto));
    }

    @ApiOperation("账款详情")
    @RequestMapping(value = "/{repalBillId}", method = RequestMethod.GET)
    public BeanResponse<RepalBillDetailVO> getRepalBillDetail(@PathVariable("repalBillId")Long repalBillId) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.getRepalBillDetail(repalBillId));
    }

    @ApiOperation("删除账款")
    @RequestMapping(value = "/deleteById", method = RequestMethod.POST)
    public BeanResponse<Boolean> deleteRepalBillById(@RequestBody DeleteRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.deleteRepalBillById(dto));
    }

    @ApiOperation("添加收款记录")
    @RequestMapping(value = "/settle", method = RequestMethod.POST)
    public BeanResponse<Boolean> settleRepalBillById(@RequestBody SettleRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.settleRepalBillById(dto));
    }

    @ApiOperation("新建账款")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public BeanResponse<Boolean> addRepalBill(@RequestBody AddRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.addRepalBill(dto));
    }

    @ApiOperation("终止账款")
    @RequestMapping(value = "/break", method = RequestMethod.POST)
    public BeanResponse<Boolean> breakRepalBill(@RequestBody BreakRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.breakRepalBill(dto));
    }

    @ApiOperation("取消终止账款")
    @RequestMapping(value = "/unBreak", method = RequestMethod.POST)
    public BeanResponse<Boolean> unBreakRepalBill(@RequestBody BreakRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.unBreakRepalBill(dto));
    }

    @ApiOperation("修改账款信息")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public BeanResponse<Boolean> updateRepalBill(@RequestBody UpdateRepalBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.updateRepalBill(dto));
    }

    @ApiOperation("账款转移")
    @RequestMapping(value = "/transfer", method = RequestMethod.POST)
    public BeanResponse<Boolean> transferBill(@RequestBody TransferBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.transferBill(dto));
    }

    @ApiOperation("回款管理列表")
    @RequestMapping(value = "/returned", method = RequestMethod.GET)
    public BeanResponse<VOPage<ReturnedBillVO>> getReturnedBillPage(ReturnedBillDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.getReturnedBillPage(dto));
    }

    @ApiOperation("回款管理详情")
    @RequestMapping(value = "/returned/detail/{repalChangeRecordId}", method = RequestMethod.GET)
    public BeanResponse<ReturnedBillDetailVO> getReturnedBillDetail(@PathVariable("repalChangeRecordId") Long repalChangeRecordId) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.getReturnedBillDetail(repalChangeRecordId));
    }

    @ApiOperation("账款评级区间历史")
    @RequestMapping(value = "/dynamic/{repalBillId}", method = RequestMethod.GET)
    public BeanResponse<DiagnosisSectionVO> getDiagnosisDynamic(@PathVariable("repalBillId")Long repalBillId) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBillService.getDiagnosisDynamic(repalBillId));
    }
}
