package com.winhc.repal.repository;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.winhc.repal.entity.RepalBill;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.winhc.repal.model.bo.*;
import com.winhc.repal.model.dto.GetRepalBillPageDTO;
import com.winhc.repal.model.vo.HomeReceivableVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @description RepalBill Mapper 接口
 * @author Generator
 * @date 2022-04-08
 */
@Repository
public interface RepalBillMapper extends BaseMapper<RepalBill> {


    /**
     * 获取账款列表
     * @param page page
     * @param billPageBO billPageBO
     * @return com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.winhc.repal.entity.RepalBill>
     * @author xda
     * @date 2022/4/13 17:44
     */
    Page<RepalBill> getRepalBillPage(Page<RepalBill> page, BillPageBO billPageBO);

    /**
     * 首页查找账本
     * @param page page
     * @param searchBillPageBO searchBillPageBO
     * @return com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.winhc.repal.entity.RepalBill>
     * @author xda
     * @date 2022/5/11 14:20
     */
    Page<RepalBill> searchPage(Page<RepalBill> page, SearchBillPageBO searchBillPageBO);


    /**
     * 获取对应类型的总金额和笔数
     * @param billPageBO billPageBO
     * @return com.winhc.repal.model.bo.TotalMoneyBO
     * @author xda
     * @date 2022/5/23 19:25
     */
    TotalMoneyBO getTotalMoneyByGroup(BillPageBO billPageBO);

    /**
     * 获取用户所负责的账款信息金额和count
     * @param userId userId
     * @param repalGroupId repalGroupId
     * @param repalGroupType repalGroupType
     * @param repalBillStatus repalBillStatus
     * @param repalBookId repalBookId
     * @return com.winhc.repal.model.bo.TotalMoneyBO
     * @author xda
     * @date 2022/4/29 14:19
     */
    TotalMoneyBO getTotalMoneyByUser(@Param("userId") Long userId,
                                     @Param("repalGroupId") Long repalGroupId,
                                     @Param("repalGroupType") Integer repalGroupType,
                                     @Param("repalBillStatus") Integer repalBillStatus,
                                     @Param("repalBookId") Long repalBookId);

    /**
     * 获取账款列表
     * @param page page
     * @param custId custId
     * @return com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.winhc.repal.entity.RepalBill>
     * @author xda
     * @date 2022/4/13 17:44
     */
    Page<RepalBill> getCustRepalBillPage(Page<RepalBill> page, @Param("custId") Long custId);

    /**
     * 查询催款汇总数据
     * @param allFlag
     * @param userId
     * @param groupId
     * @return
     */
    ManageBillInfoBO getManageBillInfo(@Param("allFlag") boolean allFlag,
                                       @Param("userId") Long userId,
                                       @Param("groupId") Long groupId,
                                       @Param("startTime") LocalDate startTime);

    /**
     * 分页查询催款列表
     * @param page
     * @param allFlag
     * @param userId
     * @param groupId
     * @param startTime
     * @return
     */
    Page<RepalManageBillPageBO> getManageBillPage(Page<RepalManageBillPageBO>page,
                                                  @Param("allFlag") boolean allFlag,
                                                  @Param("userId") Long userId,
                                                  @Param("groupId") Long groupId,
                                                  @Param("startTime") LocalDate startTime);


    /**
     * 获取账本下所有账款id
     * @param repalBookId repalBookId
     * @param userId userId
     * @return java.util.List<java.lang.Long>
     * @author xda
     * @date 2022/5/7 17:06
     */
    List<Long> getBillIdListByBookId(@Param("repalBookId") Long repalBookId,
                                     @Param("userId") Long userId);


    /**
     * 获取账本下账款的评级
     * @param repalBookId repalBookId
     * @param userId userId
     * @param start start
     * @param end end
     * @return java.util.List<java.lang.String>
     * @author xda
     * @date 2022/5/7 18:22
     */
    List<String> getBillGradeByBookId(@Param("repalBookId") Long repalBookId,
                                      @Param("userId") Long userId,
                                      @Param("start") LocalDateTime start,
                                      @Param("end") LocalDateTime end);


    /**
     * 获取账本统计信息
     * @param repalBillStatus repalBillStatus
     * @param userId userId
     * @param repalBookId repalBookId
     * @param repalBillType repalBillType
     * @param start start
     * @param end end
     * @return com.winhc.repal.model.bo.HomeReceivableBO
     * @author xda
     * @date 2022/5/9 13:22
     */
    HomeReceivableBO getBillTotal(@Param("repalBillStatus") Integer repalBillStatus,
                                  @Param("userId") Long userId,
                                  @Param("repalBookId") Long repalBookId,
                                  @Param("repalBillType") Integer repalBillType,
                                  @Param("start") LocalDateTime start,
                                  @Param("end") LocalDateTime end);
}
