package com.winhc.repal.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author: xda
 * @description: 线程池工厂，用于生产线程池，项目中所有的线程池通过此创建 目的是方便管理线程池
 * @date 2019-05-12 18:04
 */
public class ThreadPoolUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(ThreadPoolUtil.class);

    /**
     *  核心线程数
     */
    private static final Integer THREAD_NUM = 2;

    /**
     *  最大线程数
     */
    private static final Integer THREAD_MAX_NUM = 5;

    public static final ExecutorService executorService;

    /**
     *  线程编号
     */
    private static final AtomicInteger ATOMIC_INTEGER = new AtomicInteger();

    /**
     *  处理策略
     */
    public static final Thread.UncaughtExceptionHandler LOGGER_UNCAUGHT_EXCEPTION_HANDLER = (t, e) -> LOGGER.error(t.getName(), e);

    static {
        executorService = new ThreadPoolExecutor(
                THREAD_NUM, THREAD_MAX_NUM,
                TimeUnit.MINUTES.toSeconds(10),
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(),
                (r -> {
                    Thread thread = new Thread(r, "RepalService_ThreadPool_" + ATOMIC_INTEGER.getAndIncrement());
                    thread.setPriority(Thread.MAX_PRIORITY);
                    thread.setDaemon(true);
                    thread.setUncaughtExceptionHandler(LOGGER_UNCAUGHT_EXCEPTION_HANDLER);
                    return thread;
                }));
        LOGGER.info("初始化线程池成功");
    }
}
